/////////////////////////////////////////////////////////////////////////////////////////////
//
//		ARENA TABLES H     $Revision$
//
/////////////////////////////////////////////////////////////////////////////////////////////

/*  This file was forked from ParseBSA files from Stanza's WinArena open source project  */

/*  By GASNOT Marc ( kemar )
 *
 *  $Date$
 */

/*  Revision History
 *
 *  $Log$
 */



#if !defined _ARENATAB_H
#define _ARENATAB_H "@(#)arenatab.h $Revision$"



/////////////////////////////////////////////////////////////////////////////////////////////
//  HEADERS
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Standard  */
//#include	<sys/types.h>

/*  Program  */
#include	"arena.h"



/////////////////////////////////////////////////////////////////////////////////////////////
//  DEFINES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  IMG files with no headers  */
#define   ARENATAB_IMG_NOHEADER   23

/*  DAT files  */
#define   ARENATAB_DAT            25



/////////////////////////////////////////////////////////////////////////////////////////////
//  STRUCTURES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  IMG files with no headers  */
struct arenatab_images
{
	char   name[13];
	int    width;
	int    height;
};

/*  DAT files  */
struct arenatab_dat {
	char   name[13];
	int    type;
	int    var1;
	int    var2;
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  TABLES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Image files with no headers  */
struct arenatab_images   arenatab_imagesNoHeader[ARENATAB_IMG_NOHEADER] =
{
	{ "ARENARW.IMG" ,  16,  16 },   //    256 bytes
	{ "BRASS.CIF"   ,   8,  72 },   //    576 bytes
	{ "BRASS2.CIF"  ,   8,  72 },   //    576 bytes
	{ "CHARSPEL.DAT", 320, 200 },   // 64 000 bytes
	{ "CITY.IMG"    ,  16,  11 },   //    176 bytes
	{ "DITHER.IMG"  ,   8, 100 },   //    800 bytes
	{ "DITHER2.IMG" ,   8, 100 },   //    800 bytes
	{ "DUNGEON.IMG" ,  14,   8 },   //    112 bytes
	{ "DZTTAV.IMG"  ,  32,  34 },   //  1 088 bytes
	{ "FOG.TXT"     , 128, 256 },   // 32 768 bytes (Curious size, 128x128x2 anim ?)
	{ "MARBLE.CIF"  ,   4,  20 },   //     81 bytes (4x20 + 1 ? )
	{ "MARBLE2.CIF" ,   4,  20 },   //     81 bytes (4x20 + 1 ? )
	{ "NOCAMP.IMG"  ,  25,  19 },   //    475 bytes
	{ "NOSPELL.IMG" ,  25,  19 },   //    475 bytes
	{ "P1.IMG"      , 320,  53 },   // 16 960 bytes
	{ "PARCH.CIF"   ,  20, 180 },   //  3 600 bytes
	{ "POPTALK.IMG" , 320,  77 },   // 24 640 bytes
	{ "S2.IMG"      , 320,  36 },   // 11 520 bytes
	{ "SCROLL.CIF"  ,  20, 180 },   //  3 600 bytes
	{ "SLIDER.IMG"  , 289,   7 },   //  2 002 bytes
	{ "TOWN.IMG"    ,   9,  10 },   //     90 bytes
	{ "UPDOWN.IMG"  ,   8,  16 },   //    128 bytes
	{ "VILLAGE.IMG" ,   8,   8 }    //     90 bytes
};

/*  DAT Files  */
struct arenatab_dat   arenatab_datFiles[ARENATAB_DAT] = {
	{ "ARENAFNT.DAT", ARENA_FONT, 96,  9 },
	{ "ARTFACT1.DAT", ARENA_TEXT, 96,  9 },
	{ "ARTFACT2.DAT", ARENA_TEXT, 96,  9 },
	{ "CHARFNT.DAT" , ARENA_FONT, 96,  8 },
	{ "CHARSPEL.DAT", ARENA_UBIN,  0,  0 },
	{ "CLASSES.DAT" , ARENA_UBIN,  0,  0 },
	{ "EQUIP.DAT"   , ARENA_TEXT,  0,  0 },
	{ "FONT_A.DAT"  , ARENA_FONT, 96, 11 },
	{ "FONT_B.DAT"  , ARENA_FONT, 96,  6 },
	{ "FONT_C.DAT"  , ARENA_FONT, 96, 14 },
	{ "FONT_D.DAT"  , ARENA_FONT, 96,  7 },
	{ "FONT_S.DAT"  , ARENA_FONT, 96,  5 },
	{ "FONT4.DAT"   , ARENA_FONT, 96,  7 },
	{ "GUARD.DAT"   , ARENA_UBIN,  0,  0 },
	{ "JAGTEMP.DAT" , ARENA_TEXT,  0,  0 },
	{ "MUGUILD.DAT" , ARENA_TEXT,  0,  0 },
	{ "NAMECHNK.DAT", ARENA_TEXT,  0,  0 },
	{ "NAMES.DAT"   , ARENA_TEXT,  0,  0 },
	{ "NAMES2.DAT"  , ARENA_TEXT,  0,  0 },
	{ "POINTER1.DAT", ARENA_UBIN,  0,  0 },
	{ "RAND.DAT"    , ARENA_UBIN,  0,  0 },
	{ "SELLING.DAT" , ARENA_TEXT,  0,  0 },
	{ "TAVERN.DAT"  , ARENA_TEXT,  0,  0 },
	{ "TEENYFNT.DAT", ARENA_FONT, 96,  8 },
	{ "TEMPLATE.DAT", ARENA_TEXT,  0,  0 }
};

/*  INF decode  */
/////////////////////////////////////////////////////////////////////////////
//
// The following data tables are used to decode INF files.  While the INF
// files sitting the Arena directory are plain text (presumably patched, since
// the have slightly different contents from the copies in Global.BSA), those
// INF files in Global.BSA are encoded with a simple bit-hashing technique.
//
// Data encoded in the high nibble of each byte uses a rotating look-up table.
// The index into the code table changes for each byte, repeating every 16
// bytes.  This rotating offset is stored in g_RowShift[].  The second part
// of decoding the high nibble is to then apply a look-up into a 2D translation
// table using the byte's address / 16, and the 4-bit value of the high nibble.
//
// This is demonstrated in the DeobfuscateINF() function below.
//
// UPDATE: Apparently these tables can be resolved down to a single 256-byte
// array that is simply XORed against the encoded INF data to unencode it.
// I have not investigated that further, so I cannot comment.

// Note that half of the table entries are missing.  The missing entries are
// the ones that would be used to encode ASCII values 128-255.  None of the
// INF files use any of those values, nor do any of the fonts have symbols
// defined in that range.  So it's safe to say this code won't ever see one
// of those invalid values, and if it did, Arena would probably crash if it
// tried to process the file.
//
char arenatab_InverseCodeTableHigh[16][20] = {
"        67452301",
"        76543210",
"01234567        ",
"10325476        ",
"23016745        ",
"32107654        ",
"45670123        ",
"54761032        ",
"67452301        ",
"76543210        ",
"        01234567",
"        10325476",
"        23016745",
"        32107654",
"        45670123",
"        54761032"
};

// The low nibble of each byte serves as a index into a look-up table.
// The row of the table to use is based on the byte's offset, modulo 16.
// The encoding table was experimentally determined, and is stored in
// g_CodeTableLow[][].  To derive the reverse translation table, call
// MakeInverseCodeTable() one time.  Then g_InverseCodeTableLow[][]
// can be used to decode the low 4 bits of the bit stream.
//
char arenatab_CodeTableLow[16][20] =
{
	"AB89EFCD23016745",  // [address % 16] == 0
	"CDEF89AB45670123",
	"0123456789ABCDEF",
	"0123456789ABCDEF",
	"DCFE98BA54761032",  // [address % 16] == 4
	"EFCDAB8967452301",
	"EFCDAB8967452301",
	"0123456789ABCDEF",
	"23016745AB89EFCD",  // [address % 16] == 8
	"45670123CDEF89AB",
	"89ABCDEF01234567",
	"89ABCDEF01234567",
	"54761032DCFE98BA",  // [address % 16] == 12
	"67452301EFCDAB89",
	"67452301EFCDAB89",
	"89ABCDEF01234567"
};


/////////////////////////////////////////////////////////////////////////////////////////////
//  END $Source$
/////////////////////////////////////////////////////////////////////////////////////////////

#endif // _ARENATAB_H
